'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');
var fs = require('fs');
var consola = require('consola');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var consola__default = /*#__PURE__*/_interopDefaultLegacy(consola);

function isObject(val) {
  return val !== null && typeof val === "object";
}
function _defu(baseObj, defaults, namespace = ".", merger) {
  if (!isObject(defaults)) {
    return _defu(baseObj, {}, namespace, merger);
  }
  const obj = Object.assign({}, defaults);
  for (const key in baseObj) {
    if (key === "__proto__" || key === "constructor") {
      continue;
    }
    const val = baseObj[key];
    if (val === null || val === void 0) {
      continue;
    }
    if (merger && merger(obj, key, val, namespace)) {
      continue;
    }
    if (Array.isArray(val) && Array.isArray(obj[key])) {
      obj[key] = val.concat(obj[key]);
    } else if (isObject(val) && isObject(obj[key])) {
      obj[key] = _defu(val, obj[key], (namespace ? `${namespace}.` : "") + key.toString(), merger);
    } else {
      obj[key] = val;
    }
  }
  return obj;
}
function createDefu(merger) {
  return (...args) => args.reduce((p, c) => _defu(p, c, "", merger), {});
}
const defu = createDefu();

function cmkdir(dir) {
    const existed = fs__default["default"].existsSync(dir);
    if (!existed) {
        fs__default["default"].mkdirSync(dir, {
            recursive: true
        });
    }
}
function getAbsPath(p) {
    if (path__default["default"].isAbsolute(p)) {
        return p;
    }
    else {
        return path__default["default"].resolve(process.cwd(), p);
    }
}
function renderTemplete(src, replacement) {
    let t = fs__default["default"].readFileSync(src, 'utf-8');
    if (replacement) {
        const replacements = Object.entries(replacement);
        for (let i = 0; i < replacements.length; i++) {
            const [key, value] = replacements[i];
            t = t.replace(key, value);
        }
    }
    return t;
}
function withOpacityValue(variable) {
    return ({ opacityValue }) => {
        if (opacityValue === undefined) {
            return `rgb(var(${variable}))`;
        }
        return `rgb(var(${variable}) / ${opacityValue})`;
    };
}
function getKey(str) {
    return str;
}
function getJsValue(str) {
    return `withOpacityValue('${str}')`;
}
function getScssValue(str) {
    return `rgb(var(${str}))`;
}

function getOption(option) {
    var _a, _b, _c, _d;
    const filesDefault = {
        extendColors: {
            getVarName: getKey,
            getVarValue: getJsValue
        },
        variables: {
            getVarName: getKey,
            getVarValue: getScssValue
        },
        root: {
            replacement: {
            // filepath: '{{filepath}}',
            // variableName: '{{variableName}}'
            }
        },
        export: {
            replacement: {
            // filepath: '{{filepath}}',
            // variableName: '{{variableName}}'
            }
        },
        util: true
    };
    return {
        entryPoint: getAbsPath(option.entryPoint),
        outdir: (_a = option.outdir) !== null && _a !== void 0 ? _a : '.',
        files: defu(option.files, filesDefault),
        sassOptions: (_b = option.sassOptions) !== null && _b !== void 0 ? _b : {},
        outputFileSystem: (_c = option.outputFileSystem) !== null && _c !== void 0 ? _c : fs__default["default"],
        write: Boolean((_d = option.write) !== null && _d !== void 0 ? _d : true)
    };
}

// The asynchronous variants are much slower
function exposeScssVariable(exposeFilePath, options) {
    let sass;
    try {
        sass = require('sass');
    }
    catch (error) {
        consola__default["default"].error('Please install sass ! run `npm i -D sass` or `yarn add -D sass` and try again.');
        throw error;
    }
    const exposeAarry = [];
    const defaultOption = {
        functions: {
            'expose($map)': (args) => {
                const value = args[0].assertMap('map');
                const map = value.contents;
                exposeAarry.push(map);
                return value;
            }
        }
    };
    sass.compile(exposeFilePath, defu(options, defaultOption));
    return exposeAarry;
}

function generateSync(option) {
    const opt = getOption(option);
    const fs = opt.outputFileSystem;
    const { entryPoint, outdir, files, write } = opt;
    const exposeAarry = exposeScssVariable(entryPoint);
    const targetDir = path.dirname(entryPoint);
    const absOutdir = path.resolve(targetDir, outdir);
    const mergedMap = {};
    const microtaskProducer = [];
    const result = {
        js: {},
        scss: {},
        meta: []
    };
    for (let i = 0; i < exposeAarry.length; i++) {
        const map = exposeAarry[i];
        for (const [key, color] of map) {
            mergedMap[key.text] = `${color.red} ${color.green} ${color.blue}`; // ${color.alpha}
        }
    }
    const keys = Object.keys(mergedMap);
    cmkdir(absOutdir);
    consola__default["default"].info('[Output Dir]: ' + absOutdir);
    if (files.variables !== false) {
        const { getVarName, getVarValue, outfile } = files.variables;
        // scss
        // ${{ removeColorPrefix(k) }}:{{ scssFilterShadow(k) }};
        const scssResult = keys
            .map((x) => {
            return `$${getVarName(x)}:${getVarValue(x)};`;
        })
            .join('\n');
        result.scss.variables = scssResult;
        if (write) {
            microtaskProducer.push(() => {
                fs.writeFileSync(getAbsPath(outfile !== null && outfile !== void 0 ? outfile : path.resolve(absOutdir, 'variables.scss')), scssResult);
                consola__default["default"].success('[variables.scss] generate Successfully!');
            });
        }
    }
    if (files.extendColors !== false) {
        // js:
        // '{{ removeColorPrefix(k) }}':{{ jsFilterShadow(k) }},
        const filename = 'extendColors.js';
        const { getVarName, getVarValue, outfile } = files.extendColors;
        const meta = keys.map((x) => {
            return {
                name: getVarName(x),
                value: getVarValue(x)
            };
        });
        const jsResult = meta
            .map(({ name, value }) => {
            return `'${name}': ${value},`;
        })
            .join('\n    ');
        result.js.extendColors = jsResult;
        result.meta = meta;
        const extendColorsTemplete = renderTemplete(path.resolve(__dirname, `./t/js/${filename}`), {
            '/* {{placeholder}} */': jsResult
        });
        if (write) {
            microtaskProducer.push(() => {
                fs.writeFileSync(getAbsPath(outfile !== null && outfile !== void 0 ? outfile : path.resolve(absOutdir, filename)), extendColorsTemplete);
                consola__default["default"].success(`[${filename}] generate Successfully!`);
            });
        }
    }
    function handleScssFile(key, filename) {
        const file = files[key];
        if (file !== false) {
            const src = path.resolve(__dirname, `./t/scss/${filename}`);
            const dest = path.resolve(absOutdir, filename);
            if (file === true) {
                if (write) {
                    fs.copyFileSync(src, dest);
                }
            }
            else {
                const { replacement, outfile } = file;
                const content = renderTemplete(src, replacement);
                if (write) {
                    fs.writeFileSync(getAbsPath(outfile !== null && outfile !== void 0 ? outfile : dest), content, 'utf-8');
                }
            }
            write && consola__default["default"].success(`[${filename}] generate Successfully!`);
        }
    }
    microtaskProducer.push(() => handleScssFile('util', 'util.scss'));
    microtaskProducer.push(() => handleScssFile('export', 'export.scss'));
    microtaskProducer.push(() => handleScssFile('root', 'root.scss'));
    microtaskProducer.map((fn) => fn());
    return result;
}
const generate = generateSync;

function createPreset(options) {
    const defaults = {
        files: {
            root: false,
            util: false,
            variables: false,
            export: false,
            extendColors: {
                getVarName: getKey,
                getVarValue: getKey
            }
        },
        write: false
    };
    const { meta } = generateSync(defu(options, defaults));
    return {
        content: [],
        theme: {
            extend: {
                colors: meta.reduce((acc, cur) => {
                    acc[cur.name] = withOpacityValue(cur.value);
                    return acc;
                }, {})
            }
        }
    };
}

exports.createPreset = createPreset;
exports.exposeScssVariable = exposeScssVariable;
exports.generate = generate;
exports.generateSync = generateSync;
